;program pro magnetoterapii, generator pulsu
;(c) SISIE 2010
;!!bacha,
;- .equ vyhodnoti v prvnim pruchodu, nehlasi chybu phase, tj. vyraz
;  musi byt definovan pred odkazem .equ
;- sbr,cbr funguje jinak nez sbi,cbi. sbr je zkratka pro 'ORI',
;  cbr 'ANDI', zatimco sbi vyzaduje cislo bitu, sbr,cbr nepouzivat, matou.
;  vynulovat bit B: andi reg,~(1<<B),
;   nastavit bit B:  ori reg,(1<<B)
;- brsh <-> brhs = same or higher <-> half carry set
;Pro LT48 musi byt data do EEPROM zapsana od 400h po bytech, nebo
; .org $200
; .db 0,1,2,3...
;  nepouzivat .dw, protoze to assmebler otoci
;  fuse:0001_xx11
;       ||||   ^^--- cksel=11,tj.18CK+32us start, BODenabled
;       |||^-------- rstdisbl=1(pb5 je reset)
;       ||^--------- spien=0(LVPenabled)
;       |^---------- boden=0(enabled)
;       ^----------- bodlevel=0(cca 4V)
;  Kdyz se v LT48 fuse zaskrtnou, vypali se do 0. V option je nutno povolit
;  MemProt/ProgCfig. Pred programovanim je nutno precist CPU, a kalibracni
;  byte zapsat na adresu 43f LT, tj. 3f EEPROM datove.
;
;lock bity =xxxx_11xx zadna pamet neni blokovana
;
.equ	verze	=1		;verze programu

;-----------------------------------------------------------------
;program nabizi:
;--- 8 moznosti horniho (hi) kmitoctu pulsu,
;soucasny rozsah je 1.96-40Hz, kmitocty jsou v geometricke rade 16 clenu,
;frq=LO*(1,2227^(2*N+A)), kde LO=1.961Hz, N=0..7, A=0 pro LO a =1 pro HI
;pozice	frq[Hz]	per[ms] adr EEPROM, MSB first
;0	2.398	417	0,1
;1	3.584	279	2,3
;2	5.348	187	4,5
;3	8.000	125	6,7
;4	11.905	84	8,9
;5	17.857	56	0ah,0bh
;6	27.027	37	0ch,0dh
;7	40.000	25	0eh,0fh
;--- 8 moznosti dolniho (lo) kmitoctu pulsu, viz hi
;pozice	frq[Hz]	per[ms] adr EEPROM
;0	1.961	510	10h,11h
;1	2.933	341	12h,13h
;2	4.386	228	14h,15h
;3	6.536	153	16h,17h
;4	9.804	102	18h,19h
;5	14.706	68	1ah,1bh
;6	21.739	46	1ch,1dh
;7	32.714	31	1eh,1fh
;--- 8 moznosti prubehu pulsu, bitove flagy po 1 ms, tj. puls max.16ms
;0	$0003		20h,21h		...2ms puls
;1	$000f		22h,23h		...4ms puls
;2	$007f		24h,25h		...7ms puls
;3	$03ff		26h,27h		...10ms puls
;4	$fc3f		28h,29h		...6ms on, 4ms off, 6ms on
;5	$1cf7		2ah,2bh		...3on,1off,4on,2off,3on
;6	$7def		2ch,2dh		...4on,1off,4on,1off,5on
;7	$f79f		2eh,2fh		...5on,2off,4on,1off,4on
;--- 8 moznosti rozmitani kmitoctu:
;0	trvaly hi kmitocet
;1	trvaly lo kmitocet
;2	stridani hi a lo kmitoctu
;3	linearni rozmitani periody mezi hi a lo, pila rostouci perioda
;4	linearni rozmitani periody mezi hi a lo, pila klesajici perioda
;5	linearni rozmitani periody mezi hi a lo, trojuhelnik
;6	rozmitani random generatorem, rovnomerne, tj. vyssi kmitocty
;	jsou kratsi dobu, tj. spektrum energie jako u 3)-5}
;7	rozmitani random generatorem, s korekci period na priblizne
;	stejny vykon v celem pasmu od hi do lo kmitoctu
;--- 8 moznosti casovani: 0-6: po 5 minutach, (x+1)*5=5-35min,
;	7:-> timer off, konec pouze tlacitkem NEXT
;=====================================================================
;aktualni nastaveni je umisteno v EEPROM, vzdy lo 3 bity:
;30h - hi kmitocet
;31h - lo kmitocet
;32h - prubeh pulsu
;33h - rozmitani kmitoctu
;34h - casovani, tj. delka seance
;=================================================
;pouzite registry
.def	savesr	=r0	;ulozeny SR pri interruptu
.def	work0	=r30	;pracovni
.def	wint0	=r31	;pracovni
;------------- registry v klidovem stavu, tj. nastaveni - menu
.def	rgmenu	=r16	;menu - 7,6:stav, 5-3:item, 2-0:value
.def	bzmenu	=r17	;pracovni kopie rgmenu pro bzuceni
.def	bitmap	=r18	;bitovy vzorek bzucaku
			;bit7-perioda:0=>4(BZMIDF/BZNOF), 1=>6(BZHIF/BZLOF)
			;bit 6-1 (6-3) vzorek bitu, viz konstanty
.def	scount	=r19	;citac tiku T0, zakladni S perioda scanovani tlacitek
.def	tcount	=r20	;citac S period, zakladni takt bzuceni
.def	tlflag	=r21	;timer a flagy stisku tlacitek, scanovano s S periodou
			;bit4-0:timer 32*9.6ms=cca 307ms,bit5:0=>bezi timer
			;bit6,stisknuto:0=>NEXT, 1=>SET,bit7-byl stisk
.def	komut	=r22	;multiplexer bitu v rgmenu pro bzuceni
;------------- registry v pracovnim stavu, tj. generovani pulsu
.def	mincnt	=r1	;citac minut

.def	bitwhi	=r4	;pracovni kopie bitmapy (int1)
.def	bitwlo	=r5
.def	tlcthi	=r6	;citac stisknuteho tlacitka
.def	tlctlo	=r7

.def	bitehi	=r10	;bitmapa ON/OFF pulsu (16ms) z EEPROM
.def	bitelo	=r11
.def	fqhehi	=r12	;perioda hi frekvence z EEPROM
.def	fqhelo	=r13
.def	fqlehi	=r14	;perioda lo frekvence z EEPROM
.def	fqlelo	=r15
;-----------------
.def	perwhi	=r16	;pracovni kopie periody (int1)
.def	perwlo	=r17
.def	seccth	=r18	;citac vterin
.def	secctl	=r19
.def	bitcnt	=r20	;citac bitu bitmapy
;-----------------
.def	diffhi	=r21	;rozdil hi a lo frekvenci
.def	difflo	=r22
.def	randhi	=r23	;random cislo, promenna pro jiny sweep
.def	randlo	=r24
.def	perrhi	=r25	;aktualni perioda frekvence
.def	perrlo	=r26
.def	sweep	=r27	;typ rozmitani kmitoctu
.def	pomswh	=r28	;pomocne pro sweep
.def	pomswl	=r29

;konstanty pro MENU
.equ	BZLOF	=$c3		;bitovy vzorek pro LO kmitocet
.equ	BZMIDF	=$48		;bitovy vzorek pro MID kmitocet
.equ	BZHIF	=$c8		;bitovy vzorek pro HI kmitocet
.equ	BZNOF	=0		;bitovy vzorek pro pomlku
.equ	STIME	=60		;hodnota taktu S, scanovani TL, pocet tiku T0
.equ	TTIME	=8		;zakladni takt T pro delku bzuceni, nasobky S

.equ	KMINHI	=HIGH(59999)	;pocet ms za minutu
.equ	KMINLO	=LOW(59999)	;pocet ms za minutu



;obsazeni pinu
;pb0=in, pull up, ovladaci tlacitko NEXT
;pb1=out, OC1A, square 500 Hz (pro ladeni)
;pb2=in, pull up, ovladaci tlacitko SET
;pb3=out, bzucak
;pb4=out, pulsy pro vykonovy FET
;pb5=reset

.equ	TLNEXT	=0		;PB0 - tlacitko NEXT
.equ	TLSET	=2		;PB2 - tlacitko SET
.equ	BZUC	=3		;PB3 - bzucak
.equ	FETDRV	=4		;PB4 - pripojeny FET
.equ	KLID	=$25		;0,2,5 IN pull up, 1,3,4-OUT=0
.equ	BZZON	=$2d		;bzucak on
.equ	FETON	=$35		;FET on
.equ	FETBZZ	=$3d		;FET+bzucak on
;
;------------------------------------------------
.INCLUDE "include\tn15def.inc"

.cseg
.org	0
	rjmp	reset
	rjmp	reset		;int0, nepouzito
	rjmp	reset		;pin change, nepouzito
	rjmp	t1int		;T1 compA
	rjmp	reset		;T1 OVF, nepouzito
	rjmp	t0int		;T0 OVF
	rjmp	reset		;EEReady, nepouzito
	rjmp	reset		;AnalComp, nepouzito
	rjmp	reset		;ADC, nepouzito
;
;======================================================
;interrupt timeru 0
;Timer0 je pouzivan jen v klidovem stavu, tj. behem vyberu z menu
;Tik ma periodou 160us, tj. pro prescaler=1, 256/1.6 MHz=160us
;zde se ridi casovani bzucaku dle vybrane polozku menu,
;a scanuji obe tlacitka
;rutina bzucaku ocekava v registru rgmenu:
;bit 2-0 hodnota, perioda 6 tiku (000111), tj. 1.042 kHz,LOF
;bit 5-3 polozka, perioda 4 tiky (0011), tj. 1.6525 kHz, MIDF
;bit 7,6 flag nastaveni, perioda 3 tiky (001), tj. 2.083 kHz, HIF
;zakladni takt S (scanovani tlacitek) je 60 tiku, tj. 10/15/20 kmitu, tj. 9.6ms
;delky a mezery bzuceni jsou odvozeny od zakladniho taktu T = 8*S,
;tj. 8*60*160us=76.8ms.
;Log0=>1T, log1=>4T, mezera mezi bity(x)=>1T, oddelovac bloku(xxx)=>3T
;struktura:
;freq	HIFxHIFxxxMIDFxMIDFxMIDFxxxLOFxLOFxLOFxxx
;bit    6*  6     5    4    3      2   1   0
;pozn.: * znamena, ze bzuci vzdy, zbytek muze byt nahrazen mezerou, je-li
;klidovy stav, tj. bit7=0
;---------------------------------------------------
t0int:
	in	savesr,sreg
	ldi	wint0,KLID
	sbrc	bitmap,6
	ldi	wint0,BZZON
	out	PORTB,wint0	;do HW bzucaku
	bst	bitmap,3	;priprava kratke periody
	sbrc	bitmap,7
	bst	bitmap,1	;dlouha perioda
	asr	bitmap
	bld	bitmap,6	;pristi bzuceny bit
	dec	scount
	brne	t0int9		;nevyprsel S time, koncime
;je konec skupiny po S interruptech = STIME vyprsel, BZUC je v 0
	ldi	scount,STIME
	dec	tcount
	brne	tlaru1		;nevyprselo n*TTIME, scanujeme tlacitka
;n*TTIME ubehl, bzucime dalsi bit z rgmenu
;nastavime vysku (bitmap) a delku (tcount) tonu pro stav komutatoru,
;ktery v cyklu periodicky dekrementuje od 15h do 0 (krome trvaleho stavu 17h)
;Tony pro jednotlive stavy:
;	S=space (1T), Lx=lo,Mx=mid,(1/4T nebo S),
;	Hx=hi(1/4T nebo S),Ax=hi(1/4T), kde x je cislo bitu z rgmenu
;	YY=ceka na zpracovani stisku tlacitka main progrmamem
;bit1,0	11	10	01	00
;bit4,3,2:
;000	03h-L0	02h-S	01h-S	00h-S
;001	07h-L2	06h-S	05h-L1	04h-S
;010	0bh-M3	0ah-S	09h-S	08h-S
;011	0fh-M5	0eh-S	0dh-M4	0ch-S
;100	13h-H6	12h-S	11h-S	10h-S
;101	17h-YY	16h-S	15h-A6	14h-S
;
	cpi	komut,$17	;je vypnuto?
	brne	komru1		;ne
	ldi	komut,$15	;priprava akceptovani
	sbrs	tlflag,7	;uz je akceptovano tlacitko?
	rjmp	komru4		;ano
	ldi	komut,$18	;protoze se na zaver dekrementuje
	rjmp	komru2		;a formalne vysilame pomlky
komru1:
	sbrs	komut,0
	rjmp	komru2		;kazda suda (bit0=0)je kratka pomlka
	sbrs	komut,1		;pro bity 2,1=00, tj.
	sbrc	komut,2		;stavy 1,9,11h, tez
	rjmp	komru3		;neni pomlka
;vysilame pomlku
komru2:
	clr	bitmap
	ldi	tcount,TTIME
	rjmp	komru7		;hotovo
komru3:
	cpi	komut,$15	;je zacatek?
	breq	komru4		;ano
	bst	bzmenu,7	;ne, ulozime aktualni bit
	add	bzmenu,bzmenu	;a posuneme
	sbrs	rgmenu,7	;ma byt pomlka?
	rjmp	komru2		;ano
	rjmp	komru5		;ne, generujeme ton
komru4:
	mov	bzmenu,rgmenu	;obnovime stav
	add	bzmenu,bzmenu	;posuneme na bit 6
	bst	bzmenu,7	;a ulozime hodnotu
;generujeme ton, ocekava v T delku tonu a v
;komut: bity 4,3:00-LO,01-MID,10-HI vysku,
komru5:
	ldi	tcount,TTIME
	brtc	komru6
	ldi	tcount,4*TTIME
komru6:
	ldi	bitmap,BZLOF
	sbrc	komut,3
	ldi	bitmap,BZMIDF
	sbrc	komut,4
	ldi	bitmap,BZHIF
komru7:
	subi	komut,1
	brcc	komru8
	ldi	komut,$15
komru8:
;koncime vzorkovanim tlacitek
tlaru1:
	in	wint0,PINB
	com	wint0		;stisknuto = 1
	andi	wint0,(1 << TLNEXT)+(1 << TLSET)
	brne	tlaru2		;je cosi stisknuto
;neni nic stisknuto
	sbrs	tlflag,5	;neni, bezi jeste timer?
	inc	tlflag		;ano, posuneme
	rjmp	t0int9
;neco je stisknuto
tlaru2:
	sbrs	tlflag,5	;bezi jeste timer?
	rjmp	tlaru3		;bezi, resetujeme ho
;timer nebezi, je stisknuto nove tlacitko
	ldi	komut,$17	;vypiname ton
	sbrc	tlflag,7	;bylo minule akceptovano?
	rjmp	tlaru3		;ne, nechame minule
	clr	tlflag
	sbrs	wint0,TLNEXT	;je NEXT? ma prednost pred SET
	ori	tlflag,(1<<6)	;je SET
	ori	tlflag,(1<<7)	;a pro hlavni program priznak stisku
tlaru3:
	andi	tlflag,$c0	;nechame flagy urcene pro hlavni program
;koncime interrupt
t0int9:
	out	sreg,savesr	;obnovime sreg...
	reti
;===============================================================
;
;
;=====================================================
;start programu, inicializujeme system a promenne
reset:
	cli			;pro jistotu
	ldi	r31,0
	ldi     r30,KLID
	out	portb,r30
	ldi     r30,$1a		;piny 4,3,1 out
	out	ddrb,r30
	out	mcucr,r31	;pull-up en, no sleep, low level int0
	out	tccr0,r31	;timer0 stop
	out	tccr1,r31	;timer1 stop, no PWM, pin disconnected
	out	adcsr,r31	;ADC off, disabled int
	out	admux,r31	;ADC ref.disconnected
	ldi	r30,(1<<acd)
	out	acsr,r30	;komparator off, disabled int
	out	gimsk,r31	;zakaz vsech int
	out	timsk,r31
;pretahneme konstantu z EEPROM, kalibraci oscilatoru
	ldi	r30,$3f		;posledni datovy byte v EEPROM
	out	eear,r30	;je kalibracni konstanta oscilatoru
	sbi	eecr,eere	;set EEPROM Read strobe
	in	r30,eedr	;get data
	out	osccal,r30
;zapneme wdog
	wdr
	ldi	r30,$d		;512k cyklu.. 0.5s
	out	wdtcr,r30	;a wdt bezi
;inicializace pracovnich registru a spusteni menu
	clr	tlflag		;bezi timer, kdyby bylo pri resetu TL stisknute
	ldi	komut,$17	;zaciname od zacatku
	clr	bitmap
	ldi	scount,1
	ldi	tcount,1
	clr	rgmenu
	ldi	work0,0
	out	tcnt0,work0	;vynulujeme citac
	ldi	work0,$ff
	out	tifr,work0	;vynulujeme int flagy
	ldi	work0,2		;povolen int od timeru0
	out	timsk,work0
	ldi	work0,1
	out	tccr0,work0	;timer 0 bezi
	sei
;=====================================================
;hlavni smycka menu
memain:
	wdr
	sbrs	tlflag,7	;cekame na stisk tlacitka...
	rjmp	memain
	sbrs	tlflag,6	;bylo NEXT?
	rjmp	memai4		;ano
;stisknuto SET
	sbrc	rgmenu,7	;je v SET modu?
	rjmp	memai1		;ano
	ldi	rgmenu,$80	;prvni item
	rjmp	memai8		;doplnime hodnotu
memai1:
	sbrc	rgmenu,6	;je v SET VAL?
	rjmp	memai2		;ano
	ori	rgmenu,$c0	;bylo ITEM, nastavime SET VAL
	andi	rgmenu,$f8	;a prvni hodnotu (VAL=0)
	rjmp	memai9
;zapis nove hodnoty do EEPROM z rgmenu na adresu z bitu 5-3 (polozka)
;bity 2-0 (hodnota)
memai2:
	mov	work0,rgmenu
	ror	work0
	ror	work0
	ror	work0
	andi	work0,7		;offset
	ori	work0,$30	;base
	out	eear,work0
	mov	work0,rgmenu
	cli			;zakaz interruptu
	andi	work0,7		;data
	out	eedr,work0
	sbi 	EECR,EEMWE	;set master write enable
	sbi	EECR,EEWE	;set EEPROM Write strobe
	nop
	sei
memai3:
	wdr
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	memai3		;wait more
	rjmp	memai5		;a stav set ITEM
;stisknuto NEXT
memai4:
	sbrs	rgmenu,7	;je v SET modu?
	rjmp	gosetu		;ne, pobezi generovani pulsu
	sbrs	rgmenu,6	;je v SET VAL?
	rjmp	memai6		;ne, SET ITEM
;nastavujeme VAL, 0-7, je-li >=8, prepiname na SET ITEM
	mov	work0,rgmenu
	andi	work0,7		;pouze VAL
	inc	work0		;dalsi VAL
	sbrc	work0,3		;uz je >=8?
	rjmp	memai5		;ano
	andi	rgmenu,$f8
	or	rgmenu,work0
	rjmp	memai9
memai5:
	andi	rgmenu,$bf	;set ITEM
	rjmp	memai8		;doplnime hodnotu
;nastavujeme ITEM, 0-4, je-li >=5, prepiname na zakladni stav
memai6:
	mov	work0,rgmenu
	andi	work0,$38	;pouze ITEM
	subi	work0,-8	;dalsi ITEM
	cpi	work0,$28	;je item>=5?
	brlo	memai7		;ne,OK
	clr	rgmenu		;zakladni stav
	rjmp	memai9
memai7:
	andi	rgmenu,$c7	;vymaskujeme ITEM
	or	rgmenu,work0	;doplnime novou
;doplnime do rgmenu bity 2-0 (hodnota) na zaklade bitu 5-3 (polozka)
memai8:
	andi	rgmenu,$f8	;odstranime hodnotu
	mov	work0,rgmenu
	ror	work0
	ror	work0
	ror	work0
	andi	work0,7		;offset
	ori	work0,$30	;base
	out	eear,work0
	sbi	eecr,eere	;set EEPROM Read strobe
	in	work0,eedr	;get data
	andi	work0,7		;platna hodnota
	or	rgmenu,work0
memai9:
	andi	tlflag,$7f	;potvrdime stisk tlacitka
	rjmp	memain
;
;========================================================
;
; vykonny program generovani pulsu
;
;======================================================
;interrupt timeru 1
;Timer 1 je pouzivan jen behem vlastniho generovani pulsu, kdy je
;vypnut timer0
;Timer 1 bezi s periodou 1ms, tj. pro 1.6 MHz je CLK/8 a modulo 200
;zde se:
;1) inkrementuje bitcnt a dle hodnoty ridi ON/OFF, a flag konce pulsu
;2) dekrementuje tperlo, tperhi, je-li 0, nahraje se nova perioda
;3) dekrementuje secctl,seccth citac na minuty (60000),
;a mincnt citac minut (neni-li trvaly provoz, tj. bit6=1)
;generovani pulsu: bitovy vzorek je v tpuwlo,hi, a po 1 ms se posunuje,
;delka trvani pulsu je konstantni, 16ms
;---------------------------------------------------
t1int:
	in	savesr,sreg
	sbrs	mincnt,7	;je konec?
	rjmp	pulsr1		;ne
pulsr0:
	ldi	wint0,KLID	;vypiname vse
	out	PORTB,wint0
	rjmp	t1int9
pulsr1:
	mov	wint0,perwlo	;zacina nova perioda?
	or	wint0,perwhi
	ldi	wint0,KLID	;nemeni flagy
	brne	pulsr2		;ne
;zacina nova perioda
	mov	perwlo,perrlo	;pretahneme novou periodu
	mov	perwhi,perrhi
	ori	perrhi,$80	;flag: main muze generovat novou periodu
	clr	bitcnt		;zacatek pulsu
	mov	bitwlo,bitelo	;pretahneme novy vzorek pulsu
	mov	bitwhi,bitehi
	ldi	wint0,BZZON	;zapneme na 1ms bzucak
pulsr2:
	sbrc	bitcnt,4	;je >=16, tj.generuji se pulsy? (16 bitu)
	rjmp	pulsr3		;ne
	sbrc	bitwlo,0	;ma byt FET zapnuty?
	ori	wint0,FETON
	inc	bitcnt		;posuneme timer
	lsr	bitwhi		;i vzorek pulsu
	ror	bitwlo
pulsr3:
	out	PORTB,wint0
	subi	perwlo,1	;dekrementujeme zbyvajici cas periody
	sbci	perwhi,0
	subi	secctl,1	;posuneme casovac
	sbci	seccth,0
	brcc	t1int7		;neni nova minuta
	ldi	seccth,KMINHI
	ldi	secctl,KMINLO
	sbrs	mincnt,6	;neni-li bit 6=1, odecti 1,
	dec	mincnt		;je-li konec, bit 7 se nastavi
;testujeme stisk tlacitka NEXT, musi byt alespon 2s stisknute
;tj. stav citace 0x800 (8*256ms)
t1int7:
	sbic	PINB,TLNEXT	;je stisk?
	rjmp	t1int8		;ne
	inc	tlctlo
	brne	t1int9
	inc	tlcthi
	sbrs	tlcthi,3	;je uz 8?
	rjmp	t1int9		;ne
	clr	mincnt		;koncime.....
	dec	mincnt		;flag konce doby
	rjmp	pulsr0
t1int8:
	clr	tlcthi		;neni stisk, nulujeme citac
	clr	tlctlo
t1int9:
	out	sreg,savesr	;obnovime sreg...
	reti

;=================================================
;
;vykonny program generovani pulsu
gosetu:
	cli
	ldi	work0,0
	out	tccr0,work0	;timer0 stop
;pretahneme nastavene hodnoty z EEPROM
	wdr
	ldi	work0,$30	;VAL pro hi frekvenci
	out	eear,work0
	sbi	eecr,eere
	in	work0,eedr
	andi	work0,7		;platna hodnota offsetu
	lsl	work0		;tabulky po word
	out	eear,work0	;tabulka hi freq zacina od 0
	sbi	eecr,eere
	in	fqhehi,eedr
	inc	work0
	out	eear,work0
	sbi	eecr,eere
	in	fqhelo,eedr
	ldi	work0,$31	;VAL pro lo frekvenci
	out	eear,work0
	sbi	eecr,eere
	in	work0,eedr
	andi	work0,7
	lsl	work0
	subi	work0,-$10	;tabulka lo freq zacina od 0x10
	out	eear,work0
	sbi	eecr,eere
	in	fqlehi,eedr
	inc	work0
	out	eear,work0
	sbi	eecr,eere
	in	fqlelo,eedr
	ldi	work0,1
	and	fqhehi,work0	;pouze lo 9 bitu pouzijeme
	and	fqlehi,work0
	ldi	work0,$32	;VAL pro bitmapu pulsu
	out	eear,work0
	sbi	eecr,eere
	in	work0,eedr
	andi	work0,7
	lsl	work0
	subi	work0,-$20	;tabulka bitmap zacina od 0x20
	out	eear,work0
	sbi	eecr,eere
	in	bitehi,eedr
	inc	work0
	out	eear,work0
	sbi	eecr,eere
	in	bitelo,eedr
	ldi	work0,$33	;VAL pro sweep
	out	eear,work0
	sbi	eecr,eere
	in	sweep,eedr
	andi	sweep,7
	ldi	work0,$34	;VAL pro delku seance
	out	eear,work0
	sbi	eecr,eere
	in	work0,eedr
	andi	work0,7
	cpi	work0,7
	brne	goset1		;!=7, tj. nasobek 5 minut
	ldi	work0,$40	;priznak trvaleho behu
	rjmp	goset2
goset1:
	inc	work0		;pocet 5minut
	mov     wint0,work0
	add	wint0,wint0	;*2
	add	wint0,wint0	;*4
	add	work0,wint0	;*5, pocet minut
goset2:
	mov	mincnt,work0	;platny cas (1.dekrement bude vzapeti)
;vypocteme rozdil kmitoctu a zajistime, aby byl kladny, tj.hi freq > lo
	mov	diffhi,fqhehi
	mov	difflo,fqhelo
	sub	difflo,fqlelo
	sbc	diffhi,fqlehi
	brsh	goset3		;OK
;ale jen kdyz neni konstantni kmitocet (sweep=0 nebo 1)
	cpi	sweep,2
	brlo	goset3
	mov	diffhi,fqlehi	;musime zamenit kmitocty
	mov	difflo,fqlelo	;...je ve skutecnosti hi
	mov	fqlehi,fqhehi	;...je ve skutecnosti lo
	mov	fqlelo,fqhelo
	mov	fqhehi,diffhi
	mov	fqhelo,difflo
	sub	difflo,fqlelo
	sbc	diffhi,fqlehi
goset3:
	wdr
;inicializace promennych
	clr	tlcthi		;lo nechame, setrime pamet...
	ldi	bitcnt,$ff	;nepujdou pulsy
	clr	perwlo		;nastavime stejne, aby prvni dekrement casu
	clr	secctl		;zaroven spustil prvni puls, a to za
	ldi	seccth,3	;768ms, aby byl cas vse pripravit
	ldi	perwhi,3
	ldi	perrhi,$ff	;main muze nastavit novou periodu
	clr	randhi		;zaroven promenna pro ostatni druhy sweep
	ldi	randlo,$aa	;nesmi byt 0!, jinak generator nefunguje
	ldi	work0,0
	out	tcnt1,work0	;vynulujeme citac
	ldi	work0,199	;modulo citace = 200
	out	ocr1a,work0
	ldi	work0,$ff
	out	tifr,work0	;vynulujeme int flagy
	ldi	work0,$40	;povolen int od timeru1 OC1a
	out	timsk,work0
	ldi	work0,$98	;clr on OC1A,no PWM, toggle pin, CLK/8
	out	tccr1,work0	;timer 1 bezi
	sei
;=====================================================
;hlavni smycka generovani pulsu
gomain:
	wdr
	sbrs	mincnt,7	;je konec? (cas, nebo tlacitko)
	rjmp	gomai2		;ne
gomai1:
	rjmp	gomai1		;ano, cekame na wdr reset
gomai2:
	sbrs	perrhi,7	;ma byt nova perioda?
	rjmp	gomain		;jeste ne
	cpi	sweep,1
	brlo	swer00		;0=konstatni hi frekvence
	breq	swer10		;1=konstatni lo frekvence
	cpi	sweep,3
	brlo	swer20		;2=skoky mezi hi a lo
	breq	swer30		;
	cpi	sweep,5
	brlo	swer40		;4=linear, klesajici pila
	breq	swer50		;5=linear, trojuhelnik
	rjmp	swe670		;6,7=random
gomai3:
	andi	perrhi,$7f	;pro jistotu a blbe zadani...
	rjmp	gomain		;...perioda byla nastavena
;==============================================
;vykonne rutiny pripravy dalsi periody
;
;konstantni hi frekvence
swer00:
	mov	perrhi,fqhehi
	mov	perrlo,fqhelo
	rjmp	gomai3
;konstantni lo frekvence
swer10:
	mov	perrhi,fqlehi
	mov	perrlo,fqlelo
	rjmp	gomai3
;skoky mezi hi a lo frekvencemi
swer20:
	inc	randhi
	sbrc	randhi,0
	rjmp	swer00
	rjmp	swer10
;linear, rostouci pila
swer30:
	subi	randlo,-1	;freq++
	sbci	randhi,-1
	cp	fqhelo,randlo	;jsme na konci?
	cpc	fqhehi,randhi
	brcc	swer31		;ne
	mov	randhi,fqlehi	;ano, lo freq
	mov	randlo,fqlelo
swer31:
	mov	perrhi,randhi
	mov	perrlo,randlo
	rjmp	gomai3
;linear, klesajici pila
swer40:
	subi	randlo,1	;freq--
	sbci	randhi,0
	cp	randlo,fqlelo	;jsme na konci?
	cpc	randhi,fqlehi
	brcc	swer31		;ne
	mov	randhi,fqhehi	;ano, hi freq
	mov	randlo,fqhelo
	rjmp	swer31
;linear, trojuhelnik
swer50:
	sbrc	pomswl,0	;roste?
	rjmp	swer51		;ne,klesa
	subi	randlo,-1	;freq++
	sbci	randhi,-1
	cp	fqhelo,randlo	;jsme na konci?
	cpc	fqhehi,randhi
	brcc	swer31		;ne
	mov	randhi,fqhehi	;ano, hi freq
	mov	randlo,fqhelo
	inc	pomswl
	rjmp	swer31
swer51:
	subi	randlo,1	;freq--
	sbci	randhi,0
	cp	randlo,fqlelo	;jsme na konci?
	cpc	randhi,fqlehi
	brcc	swer31		;ne
	mov	randhi,fqlehi	;ano, lo freq
	mov	randlo,fqlelo
	inc	pomswl
	rjmp	swer31
;random
swe670:
;nejdriv vyrobime dalsi nahodne cislo v rozsahu 1-1023 v lo 10 bitech
;algoritmus pouziva polynom X^10 + X^7 + X^0 (481h), o delce 1023 stavu
	ldi	work0,$81		;staci lo polynomu
	add	randlo,randlo
	adc	randhi,randhi		;CRC << 1
	sbrc	randhi,2		;if( CRC & 0x400)
	eor	randlo,work0
	andi	randhi,3		;zarizneme na 10 bitu
	mov	perrlo,randlo		;a zkopirujeme pro dalsi pouziti
	mov	perrhi,randhi
	sbrs	sweep,0			;je treba korekce? (sweep=7)
	rjmp	swe673			;ne
;korekce na delku periody:
;provadi se v zavislosti na lo frekvenci, cim je lo vyssi, tim se mene koriguje
;korekci se nuluji vyssi bity, takze se nahodne cislo jakoby presouva smerem
;k nizsim hodnotam, zustava rozsah 0-511
	sbrs	perrhi,1		;RND<512?
	rjmp	swe673			;ano, nekorigujeme
	andi	perrhi,1		;rozsah 1-511
	or	fqlehi,fqlehi		;je LOF >=256?
	brne	swe673			;ano, pro LOF>=256 je rnd<1-511>
	sbrs	perrhi,1		;rnd>=256?
	rjmp	swe671			;ne, testujeme dal
	clr	perrhi
	rjmp	swe673			;pro LOF<256 a RND>256 je rnd<1-255>
swe671:
	sbrs	fqlelo,7		;je LOF >=128?
	rjmp	swe673			;ano, koncime
	sbrs	perrlo,7		;rnd>=128?
	rjmp	swe672			;ne, testujeme dal
	andi	perrlo,$7f
	rjmp	swe673			;pro LOF<128 a 128<=RND<256 je rnd<1-127>
swe672:
	sbrc	fqlelo,6		;je LOF >=64?
	andi	perrlo,$3f		;pro LOF<64 a 64<=RND<128 je rnd<1-63>
swe673:
	andi	perrhi,1		;rozsah 1-511
;rozprostreme nahodna cisla 1-511 do intervalu <LOF,HIF> nasobenim
;PERR(9bit)*DIFF(9bit) -> PERR a delenim 512
	clr	pomswh		;clear 2 highest bytes of result
	clr	pomswl
	ldi	work0,9		;init loop counter
	lsr	perrhi
	ror	perrlo
swe680:
	brcc	swe681		;if bit 0 of multiplier set
	add	pomswl,difflo	;add multiplicand Low to byte 2 of res
	adc	pomswh,diffhi	;add multiplicand high to byte 3 of res
swe681:
	ror	pomswh		;shift right result byte 3
	ror	pomswl		;rotate right result byte 2
	ror	perrhi		;rotate result byte 1 and multiplier High
	ror	perrlo		;rotate result byte 0 and multiplier Low
	dec	work0		;decrement loop counter
	brne	swe680		;if not done, loop more
;vysledek je v pomswh(bit 17),pomswl(bity16-9),perrhi(bity8-1) a perrlo(bit0),
;bereme pouze hornich 9 bitu (delime 512)
	mov	perrhi,pomswh	;msb na miste
	mov	perrlo,pomswl	;lsb tez
;a pricteme lo kmitocet
	add	perrlo,fqlelo
	adc	perrhi,fqlehi
	rjmp	gomai3
;

;================================================
;
;************************************************
;
;	****************
;	* datova pamet *
;	****************
;

;posledni datovy byte $3f je kalibracni konstanta oscilatoru

;@@@ vlastni programator
;.org $200
; .db 0,0			;co nejmensi strida
;.org $21f
; .db $ff,$60			;kalibracni konstanta oscilatoru

.eseg
.org	0
;program nabizi:
;--- 8 moznosti horniho (hi) kmitoctu pulsu,
;soucasny rozsah je 1.96-40Hz, kmitocty jsou v geometricke rade 16 clenu,
;frq=LO*(1,2227^(2*N+A)), kde LO=1.961Hz, N=0..7, A=0 pro LO a =1 pro HI
;pozice	frq[Hz]	per[ms] adr EEPROM, MSB first
;0	2.398	417	0,1
;1	3.584	279	2,3
;2	5.348	187	4,5
;3	8.000	125	6,7
;4	11.905	84	8,9
;5	17.857	56	0ah,0bh
;6	27.027	37	0ch,0dh
;7	40.000	25	0eh,0fh
.db	HIGH(417),LOW(417),HIGH(279),LOW(279),0,187,0,125
.db	0,84,0,56,0,37,0,25
;--- 8 moznosti dolniho (lo) kmitoctu pulsu, viz hi
;pozice	frq[Hz]	per[ms] adr EEPROM
;0	1.961	510	10h,11h
;1	2.933	341	12h,13h
;2	4.386	228	14h,15h
;3	6.536	153	16h,17h
;4	9.804	102	18h,19h
;5	14.706	68	1ah,1bh
;6	21.739	46	1ch,1dh
;7	32.714	31	1eh,1fh
.db	HIGH(510),LOW(510),HIGH(341),LOW(341),0,228,0,153
.db	0,102,0,68,0,46,0,31
;--- 8 moznosti prubehu pulsu, bitove flagy po 1 ms, tj. puls max.16ms
;0	$0003		20h,21h		...2ms puls
;1	$000f		22h,23h		...4ms puls
;2	$007f		24h,25h		...7ms puls
;3	$03ff		26h,27h		...10ms puls
;4	$fc3f		28h,29h		...6ms on, 4ms off, 6ms on
;5	$1cf7		2ah,2bh		...3on,1off,4on,2off,3on
;6	$7def		2ch,2dh		...4on,1off,4on,1off,5on
;7	$f79f		2eh,2fh		...5on,2off,4on,1off,4on
.db	0,3,0,$f,0,$7f,$3,$ff,$fc,$3f,$1c,$f7,$7d,$ef,$f7,$9f
;--- 8 moznosti rozmitani kmitoctu:
;0	trvaly hi kmitocet
;1	trvaly lo kmitocet
;2	stridani hi a lo kmitoctu
;3	linearni rozmitani periody mezi hi a lo, pila rostouci perioda
;4	linearni rozmitani periody mezi hi a lo, pila klesajici perioda
;5	linearni rozmitani periody mezi hi a lo, trojuhelnik
;6	rozmitani random generatorem, rovnomerne, tj. vyssi kmitocty
;	jsou kratsi dobu, tj. spektrum energie jako u 3)-5}
;7	rozmitani random generatorem, s korekci period na priblizne
;	stejny vykon v celem pasmu od hi do lo kmitoctu
;--- 8 moznosti casovani: 0-6: po 5 minutach, (x+1)*5=5-35min,
;	7:-> timer off, konec pouze tlacitkem NEXT
;=====================================================================
;aktualni nastaveni je umisteno v EEPROM, vzdy lo 3 bity:
;30h - hi kmitocet
;31h - lo kmitocet
;32h - prubeh pulsu
;33h - rozmitani kmitoctu
;34h - casovani, tj. delka seance
;=================================================
.org	$30
.db	4,3,2,1,0

.org	$3f
.db	$60
